@@ -1,3 +1,21 @@
+/*
+ * This file is part of dsp.
+ *
+ * Copyright (c) 2014-2024 Michael Barbour <barbour.michael.0@gmail.com>
+ *
+ * Permission to use, copy, modify, and distribute this software for any
+ * purpose with or without fee is hereby granted, provided that the above
+ * copyright notice and this permission notice appear in all copies.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
+ * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
+ * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
+ * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
+ * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
+ * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+*/
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <stdint.h>
@@ -11,8 +29,8 @@ struct ffmpeg_state {
 	AVFormatContext *container;
 	AVCodecContext *cc;
 	AVFrame *frame;
-	void (*read_func)(char *, sample_t *, ssize_t);
-	void (*readp_func)(char **, sample_t *, int, ssize_t, ssize_t);
+	void (*read_func)(void *, sample_t *, ssize_t);
+	void (*readp_func)(void **, sample_t *, int, ssize_t, ssize_t);
 	int planar, bytes, stream_index, got_frame;
 	ssize_t frame_pos;
 	int64_t last_ts;
@@ -23,9 +41,9 @@ static int av_initialized = 0;
 
 /* Planar sample conversion functions */
 
-static void read_buf_u8p(char **in, sample_t *out, int channels, ssize_t start, ssize_t s)
+static void read_buf_u8p(void **in, sample_t *out, int channels, ssize_t start, ssize_t s)
 {
-	unsigned char **inn = (unsigned char **) in;
+	uint8_t **inn = (uint8_t **) in;
 	int c = channels;
 	ssize_t out_s = s * channels;
 	while (s-- > 0) {
@@ -35,9 +53,9 @@ static void read_buf_u8p(char **in, sample_t *out, int channels, ssize_t start,
 	}
 }
 
-static void read_buf_s16p(char **in, sample_t *out, int channels, ssize_t start, ssize_t s)
+static void read_buf_s16p(void **in, sample_t *out, int channels, ssize_t start, ssize_t s)
 {
-	signed short **inn = (signed short **) in;
+	int16_t **inn = (int16_t **) in;
 	int c = channels;
 	ssize_t out_s = s * channels;
 	while (s-- > 0) {
@@ -47,9 +65,9 @@ static void read_buf_s16p(char **in, sample_t *out, int channels, ssize_t start,
 	}
 }
 
-static void read_buf_s32p(char **in, sample_t *out, int channels, ssize_t start, ssize_t s)
+static void read_buf_s32p(void **in, sample_t *out, int channels, ssize_t start, ssize_t s)
 {
-	signed int **inn = (signed int **) in;
+	int32_t **inn = (int32_t **) in;
 	int c = channels;
 	ssize_t out_s = s * channels;
 	while (s-- > 0) {
@@ -59,7 +77,7 @@ static void read_buf_s32p(char **in, sample_t *out, int channels, ssize_t start,
 	}
 }
 
-static void read_buf_floatp(char **in, sample_t *out, int channels, ssize_t start, ssize_t s)
+static void read_buf_floatp(void **in, sample_t *out, int channels, ssize_t start, ssize_t s)
 {
 	float **inn = (float **) in;
 	int c = channels;
@@ -71,7 +89,7 @@ static void read_buf_floatp(char **in, sample_t *out, int channels, ssize_t star
 	}
 }
 
-static void read_buf_doublep(char **in, sample_t *out, int channels, ssize_t start, ssize_t s)
+static void read_buf_doublep(void **in, sample_t *out, int channels, ssize_t start, ssize_t s)
 {
 	double **inn = (double **) in;
 	int c = channels;
@@ -139,10 +157,10 @@ ssize_t ffmpeg_read(struct codec *c, sample_t *buf, ssize_t frames)
 		if (state->got_frame) {
 			avail = (avail > frames - buf_pos) ? frames - buf_pos : avail;
 			if (state->planar)
-				state->readp_func((char **) state->frame->extended_data, &buf[buf_pos * c->channels],
+				state->readp_func((void **) state->frame->extended_data, &buf[buf_pos * c->channels],
 					c->channels, state->frame_pos, avail);
 			else
-				state->read_func((char *) &state->frame->extended_data[0][state->frame_pos * state->bytes * c->channels],
+				state->read_func(&state->frame->extended_data[0][state->frame_pos * state->bytes * c->channels],
 					&buf[buf_pos * c->channels], avail * c->channels);
 			buf_pos += avail;
 			state->frame_pos += avail;
@@ -152,11 +170,6 @@ ssize_t ffmpeg_read(struct codec *c, sample_t *buf, ssize_t frames)
 	return buf_pos;
 }
 
-ssize_t ffmpeg_write(struct codec *c, sample_t *buf, ssize_t frames)
-{
-	return 0;
-}
-
 ssize_t ffmpeg_seek(struct codec *c, ssize_t pos)
 {
 	AVStream *st;
@@ -203,7 +216,7 @@ void ffmpeg_destroy(struct codec *c)
 	free((char *) c->type);
 }
 
-struct codec * ffmpeg_codec_init(const char *path, const char *type, const char *enc, int fs, int channels, int endian, int mode)
+struct codec * ffmpeg_codec_init(const struct codec_params *p)
 {
 	int i, err;
 	struct ffmpeg_state *state = NULL;
@@ -221,9 +234,8 @@ struct codec * ffmpeg_codec_init(const char *path, const char *type, const char
 
 	/* open input and find stream info */
 	state = calloc(1, sizeof(struct ffmpeg_state));
-	if ((err = avformat_open_input(&state->container, path, NULL, NULL)) < 0) {
-		LOG_FMT(LL_OPEN_ERROR, "%s: error: failed to open %s: %s: %s", codec_name,
-			(mode == CODEC_MODE_WRITE) ? "output" : "input", path, av_err2str(err));
+	if ((err = avformat_open_input(&state->container, p->path, NULL, NULL)) < 0) {
+		LOG_FMT(LL_OPEN_ERROR, "%s: error: failed to open input: %s: %s", codec_name, p->path, av_err2str(err));
 		goto fail;
 	}
 	if ((err = avformat_find_stream_info(state->container, NULL)) < 0) {
@@ -269,7 +281,7 @@ struct codec * ffmpeg_codec_init(const char *path, const char *type, const char
 
 	c = calloc(1, sizeof(struct codec));
 	i = strlen(codec->name) + 8;
-	c->path = path;
+	c->path = p->path;
 	c->type = calloc(1, i);
 	snprintf((char *) c->type, i, "ffmpeg/%s", codec->name);
 	c->enc = av_get_sample_fmt_name(state->cc->sample_fmt);
@@ -283,21 +295,21 @@ struct codec * ffmpeg_codec_init(const char *path, const char *type, const char
 	case AV_SAMPLE_FMT_U8:
 	case AV_SAMPLE_FMT_U8P:
 		c->prec = 8;
-		c->can_dither = 1;
+		c->hints |= CODEC_HINT_CAN_DITHER;
 		state->read_func = read_buf_u8;
 		state->readp_func = read_buf_u8p;
 		break;
 	case AV_SAMPLE_FMT_S16:
 	case AV_SAMPLE_FMT_S16P:
 		c->prec = 16;
-		c->can_dither = 1;
+		c->hints |= CODEC_HINT_CAN_DITHER;
 		state->read_func = read_buf_s16;
 		state->readp_func = read_buf_s16p;
 		break;
 	case AV_SAMPLE_FMT_S32:
 	case AV_SAMPLE_FMT_S32P:
 		c->prec = 32;
-		c->can_dither = 1;
+		c->hints |= CODEC_HINT_CAN_DITHER;
 		state->read_func = read_buf_s32;
 		state->readp_func = read_buf_s32p;
 		break;
@@ -324,7 +336,6 @@ struct codec * ffmpeg_codec_init(const char *path, const char *type, const char
 	else
 		c->frames = -1;
 	c->read = ffmpeg_read;
-	c->write = ffmpeg_write;
 	c->seek = ffmpeg_seek;
 	c->delay = ffmpeg_delay;
 	c->drop = ffmpeg_drop;
@@ -339,7 +350,7 @@ struct codec * ffmpeg_codec_init(const char *path, const char *type, const char
 	if (state->frame)
 		av_frame_free(&state->frame);
 	if (state->cc)
-		avcodec_close(state->cc);
+		avcodec_free_context(&state->cc);
 	if (state->container)
 		avformat_close_input(&state->container);
 	free(state);
